@@ -8,184 +8,13 @@ import (
 	kapi "k8s.io/api/core/v1"
 	"k8s.io/client-go/tools/cache"
 
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/ovn"
-	"github.com/openvswitch/ovn-kubernetes/go-controller/pkg/util"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	"github.com/openshift/origin/pkg/util/netutils"
 	"github.com/sirupsen/logrus"
 )
 
-// RebuildOVNDatabase rebuilds db if HA option is enabled and OVN DB
-// doesn't exist. First It updates k8s nodes by creating a logical
-// switch for each node. Then it reads all resources from k8s and
-// creates needed resources in OVN DB. Last, it updates master node
-// ip in default namespace to trigger event on node.
-func (cluster *OvnClusterController) RebuildOVNDatabase(
-	masterNodeName string, oc *ovn.Controller) error {
-	logrus.Debugf("Rebuild OVN database for cluster nodes")
-	var err error
-	ipChange, err := cluster.checkMasterIPChange(masterNodeName)
-	if err != nil {
-		logrus.Errorf("Error when checking Master Node IP Change: %v", err)
-		return err
-	}
-
-	// If OvnHA options is enabled, make sure default namespace has the
-	// annotation of current cluster master node's overlay IP address
-	logrus.Debugf("cluster.OvnHA: %t", cluster.OvnHA)
-	if cluster.OvnHA && ipChange {
-		logrus.Debugf("HA is enabled and DB doesn't exist!")
-		// Rebuild OVN DB for the k8s nodes
-		err = cluster.UpdateDBForKubeNodes(masterNodeName)
-		if err != nil {
-			return err
-		}
-		// Rebuild OVN DB for the k8s namespaces and all the resource
-		// objects inside the namespace including pods and network
-		// policies
-		err = cluster.UpdateKubeNsObjects(oc)
-		if err != nil {
-			return err
-		}
-		// Update master node IP annotation on default namespace
-		err = cluster.UpdateMasterNodeIP(masterNodeName)
-		if err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
-// UpdateDBForKubeNodes rebuilds ovn db for k8s nodes
-func (cluster *OvnClusterController) UpdateDBForKubeNodes(
-	masterNodeName string) error {
-	nodes, err := cluster.Kube.GetNodes()
-	if err != nil {
-		logrus.Errorf("Failed to obtain k8s nodes: %v", err)
-		return err
-	}
-	for _, node := range nodes.Items {
-		subnetStr, ok := node.Annotations[OvnHostSubnet]
-		if ok {
-			ip, subnet, err := net.ParseCIDR(subnetStr)
-			if err != nil {
-				return fmt.Errorf("Failed to parse subnet %s: %v", subnetStr, err)
-			}
-			subnet.IP = util.NextIP(ip)
-			err = cluster.ensureNodeLogicalNetwork(node.Name, subnet)
-			if err != nil {
-				return err
-			}
-		}
-	}
-	return nil
-}
-
-// UpdateKubeNsObjects rebuilds ovn db for k8s namespaces
-// and pods/networkpolicies inside the namespaces.
-func (cluster *OvnClusterController) UpdateKubeNsObjects(
-	oc *ovn.Controller) error {
-	namespaces, err := cluster.Kube.GetNamespaces()
-	if err != nil {
-		logrus.Errorf("Failed to get k8s namespaces: %v", err)
-		return err
-	}
-	for _, ns := range namespaces.Items {
-		oc.AddNamespace(&ns)
-		pods, err := cluster.Kube.GetPods(ns.Name)
-		if err != nil {
-			logrus.Errorf("Failed to get k8s pods: %v", err)
-			return err
-		}
-		for _, pod := range pods.Items {
-			oc.AddLogicalPortWithIP(&pod)
-		}
-		endpoints, err := cluster.Kube.GetEndpoints(ns.Name)
-		if err != nil {
-			logrus.Errorf("Failed to get k8s endpoints: %v", err)
-			return err
-		}
-		for _, ep := range endpoints.Items {
-			er := oc.AddEndpoints(&ep)
-			if er != nil {
-				logrus.Errorf("Error adding endpoints: %v", er)
-				return er
-			}
-		}
-		policies, err := cluster.Kube.GetNetworkPolicies(ns.Name)
-		if err != nil {
-			logrus.Errorf("Failed to get k8s network policies: %v", err)
-			return err
-		}
-		for _, policy := range policies.Items {
-			oc.AddNetworkPolicy(&policy)
-		}
-	}
-	return nil
-}
-
-// UpdateMasterNodeIP add annotations of master node IP on
-// default namespace
-func (cluster *OvnClusterController) UpdateMasterNodeIP(
-	masterNodeName string) error {
-	masterNodeIP, err := netutils.GetNodeIP(masterNodeName)
-	if err != nil {
-		return fmt.Errorf("Failed to obtain local IP from master node "+
-			"%q: %v", masterNodeName, err)
-	}
-
-	defaultNs, err := cluster.Kube.GetNamespace(DefaultNamespace)
-	if err != nil {
-		return fmt.Errorf("Failed to get default namespace: %v", err)
-	}
-
-	// Get overlay ip on OVN master node from default namespace. If it
-	// doesn't have it or the IP address is different than the current one,
-	// set it with current master overlay IP.
-	masterIP, ok := defaultNs.Annotations[MasterOverlayIP]
-	if !ok || masterIP != masterNodeIP {
-		err := cluster.Kube.SetAnnotationOnNamespace(defaultNs, MasterOverlayIP,
-			masterNodeIP)
-		if err != nil {
-			return fmt.Errorf("Failed to set %s=%s on namespace %s: %v",
-				MasterOverlayIP, masterNodeIP, defaultNs.Name, err)
-		}
-	}
-
-	return nil
-}
-
-func (cluster *OvnClusterController) checkMasterIPChange(
-	masterNodeName string) (bool, error) {
-	// Check DB existence by checking if the default namespace annotated
-	// IP address is the same as the master node IP. If annotated IP
-	// address is different, we assume that the ovn db is crashed on the
-	// old node and needs to be rebuilt on the new master node.
-	masterNodeIP, err := netutils.GetNodeIP(masterNodeName)
-	if err != nil {
-		return false, fmt.Errorf("Failed to obtain local IP from master "+
-			"node %q: %v", masterNodeName, err)
-	}
-
-	defaultNs, err := cluster.Kube.GetNamespace(DefaultNamespace)
-	if err != nil {
-		return false, fmt.Errorf("Failed to get default namespace: %v", err)
-	}
-
-	// Get overlay ip on OVN master node from default namespace. If the IP
-	// address is different than master node IP, return true. Else, return
-	// false.
-	masterIP := defaultNs.Annotations[MasterOverlayIP]
-	logrus.Debugf("Master IP: %s, Annotated IP: %s", masterNodeIP, masterIP)
-	if masterIP != masterNodeIP {
-		logrus.Debugf("Detected Master node IP is different than default " +
-			"namespae annotated IP.")
-		return true, nil
-	}
-	return false, nil
-}
-
 // StartClusterMaster runs a subnet IPAM and a controller that watches arrival/departure
 // of nodes in the cluster
 // On an addition to the cluster (node create), a new subnet is created for it that will translate
@@ -212,7 +41,7 @@ func (cluster *OvnClusterController) StartClusterMaster(masterNodeName string) e
 	// NewSubnetAllocator is a subnet IPAM, which takes a CIDR (first argument)
 	// and gives out subnets of length 'hostSubnetLength' (second argument)
 	// but omitting any that exist in 'subrange' (third argument)
-	for _, clusterEntry := range cluster.ClusterIPNet {
+	for _, clusterEntry := range config.Default.ClusterSubnets {
 		subrange := make([]string, 0)
 		for _, allocatedRange := range alreadyAllocated {
 			firstAddress, _, err := net.ParseCIDR(allocatedRange)
@@ -287,8 +116,12 @@ func (cluster *OvnClusterController) SetupMaster(masterNodeName string) error {
 	}
 
 	// Create a logical switch called "join" that will be used to connect gateway routers to the distributed router.
-	// The "join" will be allocated IP addresses in the range 100.64.0.0/16.
-	stdout, stderr, err = util.RunOVNNbctl("--may-exist", "ls-add", "join")
+	// The "join" switch will be allocated IP addresses in the range 100.64.0.0/16.
+	const joinSubnet string = "100.64.0.1/16"
+	joinIP, joinCIDR, _ := net.ParseCIDR(joinSubnet)
+	stdout, stderr, err = util.RunOVNNbctl("--may-exist", "ls-add", "join",
+		"--", "set", "logical_switch", "join", fmt.Sprintf("other-config:subnet=%s", joinCIDR.String()),
+		"--", "set", "logical_switch", "join", fmt.Sprintf("other-config:exclude_ips=%s", joinIP.String()))
 	if err != nil {
 		logrus.Errorf("Failed to create logical switch called \"join\", stdout: %q, stderr: %q, error: %v", stdout, stderr, err)
 		return err
@@ -303,8 +136,7 @@ func (cluster *OvnClusterController) SetupMaster(masterNodeName string) error {
 	if routerMac == "" {
 		routerMac = util.GenerateMac()
 		stdout, stderr, err = util.RunOVNNbctl("--", "--may-exist", "lrp-add", OvnClusterRouter,
-			"rtoj-"+OvnClusterRouter, routerMac, "100.64.0.1/16", "--", "set", "logical_router_port",
-			"rtoj-"+OvnClusterRouter, "external_ids:connect_to_join=yes")
+			"rtoj-"+OvnClusterRouter, routerMac, joinSubnet)
 		if err != nil {
 			logrus.Errorf("Failed to add logical router port rtoj-%v, stdout: %q, stderr: %q, error: %v",
 				OvnClusterRouter, stdout, stderr, err)
@@ -322,15 +154,6 @@ func (cluster *OvnClusterController) SetupMaster(masterNodeName string) error {
 		return err
 	}
 
-	// Create a lock for gateway-init to co-ordinate.
-	stdout, stderr, err = util.RunOVNNbctl("--", "set", "nb_global", ".",
-		"external-ids:gateway-lock=\"\"")
-	if err != nil {
-		logrus.Errorf("Failed to create lock for gateways, "+
-			"stdout: %q, stderr: %q, error: %v", stdout, stderr, err)
-		return err
-	}
-
 	return nil
 }
 
@@ -348,30 +171,6 @@ func parseNodeHostSubnet(node *kapi.Node) (*net.IPNet, error) {
 	return subnet, nil
 }
 
-func (cluster *OvnClusterController) ensureNodeHostSubnet(node *kapi.Node) (*net.IPNet, *netutils.SubnetAllocator, error) {
-	// Do not create a subnet if the node already has a subnet
-	subnet, _ := parseNodeHostSubnet(node)
-	if subnet != nil {
-		return subnet, nil, nil
-	}
-
-	// Create new subnet
-	for _, possibleSubnet := range cluster.masterSubnetAllocatorList {
-		sn, err := possibleSubnet.GetNetwork()
-		if err == netutils.ErrSubnetAllocatorFull {
-			// Current subnet exhausted, check next possible subnet
-			continue
-		} else if err != nil {
-			return nil, nil, fmt.Errorf("Error allocating network for node %s: %v", node.Name, err)
-		}
-
-		// Success
-		logrus.Infof("Allocated node %s HostSubnet %s", node.Name, sn.String())
-		return sn, possibleSubnet, nil
-	}
-	return nil, nil, fmt.Errorf("error allocating network for node %s: No more allocatable ranges", node.Name)
-}
-
 func (cluster *OvnClusterController) ensureNodeLogicalNetwork(nodeName string, hostsubnet *net.IPNet) error {
 	ip := util.NextIP(hostsubnet.IP)
 	n, _ := hostsubnet.Mask.Size()
@@ -393,9 +192,15 @@ func (cluster *OvnClusterController) ensureNodeLogicalNetwork(nodeName string, h
 		return err
 	}
 
+	// Skip the second address of the LogicalSwitch's subnet since we set it aside for the
+	// management port on that node.
+	secondIP := util.NextIP(ip)
+
 	// Create a logical switch and set its subnet.
 	stdout, stderr, err := util.RunOVNNbctl("--", "--may-exist", "ls-add", nodeName,
-		"--", "set", "logical_switch", nodeName, "other-config:subnet="+hostsubnet.String(), "external-ids:gateway_ip="+firstIP)
+		"--", "set", "logical_switch", nodeName, "other-config:subnet="+hostsubnet.String(),
+		"other-config:exclude_ips="+secondIP.String(),
+		"external-ids:gateway_ip="+firstIP)
 	if err != nil {
 		logrus.Errorf("Failed to create a logical switch %v, stdout: %q, stderr: %q, error: %v", nodeName, stdout, stderr, err)
 		return err
@@ -432,15 +237,28 @@ func (cluster *OvnClusterController) ensureNodeLogicalNetwork(nodeName string, h
 }
 
 func (cluster *OvnClusterController) addNode(node *kapi.Node) (err error) {
-	var hostsubnet *net.IPNet
+	hostsubnet, _ := parseNodeHostSubnet(node)
+	if hostsubnet != nil {
+		// Node already has subnet assigned; ensure its logical network is set up
+		return cluster.ensureNodeLogicalNetwork(node.Name, hostsubnet)
+	}
+
+	// Node doesn't have a subnet assigned; reserve a new one for it
 	var subnetAllocator *netutils.SubnetAllocator
-	hostsubnet, subnetAllocator, err = cluster.ensureNodeHostSubnet(node)
-	if err != nil {
-		return err
+	for _, subnetAllocator = range cluster.masterSubnetAllocatorList {
+		hostsubnet, err = subnetAllocator.GetNetwork()
+		if err == netutils.ErrSubnetAllocatorFull {
+			// Current subnet exhausted, check next possible subnet
+			continue
+		} else if err != nil {
+			return fmt.Errorf("Error allocating network for node %s: %v", node.Name, err)
+		}
+		logrus.Infof("Allocated node %s HostSubnet %s", node.Name, hostsubnet.String())
 	}
 
 	defer func() {
-		if err != nil && subnetAllocator != nil {
+		// Release the allocation on error
+		if err != nil {
 			_ = subnetAllocator.ReleaseNetwork(hostsubnet)
 		}
 	}()
@@ -456,7 +274,8 @@ func (cluster *OvnClusterController) addNode(node *kapi.Node) (err error) {
 	// proceed with their initialization
 	err = cluster.Kube.SetAnnotationOnNode(node, OvnHostSubnet, hostsubnet.String())
 	if err != nil {
-		logrus.Errorf("Failed to set node %s host subnet annotation: %v", node.Name, hostsubnet.String())
+		logrus.Errorf("Failed to set node %s host subnet annotation to %q: %v",
+			node.Name, hostsubnet.String(), err)
 		return err
 	}
 
@@ -522,7 +341,9 @@ func (cluster *OvnClusterController) syncNodes(nodes []interface{}) {
 	}
 
 	// We only deal with cleaning up nodes that shouldn't exist here, since
-	// watchNodes() will be called for all existing nodes at startup anyway
+	// watchNodes() will be called for all existing nodes at startup anyway.
+	// Note that this list will include the 'join' cluster switch, which we
+	// do not want to delete.
 	nodeSwitches, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading",
 		"--columns=name,other-config", "find", "logical_switch", "other-config:subnet!=_")
 	if err != nil {
@@ -536,6 +357,10 @@ func (cluster *OvnClusterController) syncNodes(nodes []interface{}) {
 		if len(items) != 2 || len(items[0]) == 0 {
 			continue
 		}
+		if items[0] == "join" {
+			// Don't delete the cluster switch
+			continue
+		}
 		if _, ok := foundNodes[items[0]]; ok {
 			// node still exists, no cleanup to do
 			continue
