@@ -3,6 +3,7 @@ package config
 import (
 	"fmt"
 	"io/ioutil"
+	"net"
 	"os"
 	"path/filepath"
 	"strings"
@@ -10,9 +11,8 @@ import (
 
 	"github.com/urfave/cli"
 	kexec "k8s.io/utils/exec"
-	fakeexec "k8s.io/utils/exec/testing"
 
-	ovntest "github.com/openvswitch/ovn-kubernetes/go-controller/pkg/testing"
+	ovntest "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/testing"
 
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -121,6 +121,81 @@ func createTempFile(name string) (string, error) {
 	return fname, nil
 }
 
+func createTempFileContent(name, value string) (string, error) {
+	fname := filepath.Join(tmpDir, name)
+	if err := ioutil.WriteFile(fname, []byte(value), 0644); err != nil {
+		return "", err
+	}
+	return fname, nil
+}
+
+// writeTestConfigFile writes out a config file with well-known options but
+// allows specific fields to be overridden by the testcase
+func writeTestConfigFile(path string, overrides ...string) error {
+	const defaultData string = `[default]
+mtu=1500
+conntrack-zone=64321
+cluster-subnets=10.129.0.0/14/23
+
+[kubernetes]
+kubeconfig=/path/to/kubeconfig
+apiserver=https://1.2.3.4:6443
+token=TG9yZW0gaXBzdW0gZ
+cacert=/path/to/kubeca.crt
+service-cidr=172.18.0.0/24
+
+[logging]
+loglevel=5
+logfile=/var/log/ovnkube.log
+
+[cni]
+conf-dir=/etc/cni/net.d22
+plugin=ovn-k8s-cni-overlay22
+
+[ovnnorth]
+address=ssl://1.2.3.4:6641
+client-privkey=/path/to/nb-client-private.key
+client-cert=/path/to/nb-client.crt
+client-cacert=/path/to/nb-client-ca.crt
+
+[ovnsouth]
+address=ssl://1.2.3.4:6642
+client-privkey=/path/to/sb-client-private.key
+client-cert=/path/to/sb-client.crt
+client-cacert=/path/to/sb-client-ca.crt
+
+[gateway]
+mode=shared
+interface=eth1
+next-hop=1.3.4.5
+vlan-id=10
+nodeport=false
+`
+
+	var newData string
+	for _, line := range strings.Split(defaultData, "\n") {
+		equalsPos := strings.Index(line, "=")
+		if equalsPos >= 0 {
+			for _, override := range overrides {
+				if strings.HasPrefix(override, line[:equalsPos+1]) {
+					line = override
+					break
+				}
+			}
+		}
+		newData += line + "\n"
+	}
+	return ioutil.WriteFile(path, []byte(newData), 0644)
+}
+
+func mustParseCIDR(cidr string) *net.IPNet {
+	_, net, err := net.ParseCIDR(cidr)
+	if err != nil {
+		panic("bad CIDR string constant " + cidr)
+	}
+	return net
+}
+
 var _ = Describe("Config Operations", func() {
 	var app *cli.App
 	var cfgFile *os.File
@@ -130,6 +205,7 @@ var _ = Describe("Config Operations", func() {
 	if tmpErr != nil {
 		GinkgoT().Errorf("failed to create tempdir: %v", tmpErr)
 	}
+	tmpDir += "/"
 
 	BeforeEach(func() {
 		// Restore global default values before each testcase
@@ -150,7 +226,7 @@ var _ = Describe("Config Operations", func() {
 
 	It("uses expected defaults", func() {
 		app.Action = func(ctx *cli.Context) error {
-			cfgPath, err := InitConfig(ctx, kexec.New(), nil)
+			cfgPath, err := InitConfigSa(ctx, kexec.New(), tmpDir, nil)
 			Expect(err).NotTo(HaveOccurred())
 			Expect(cfgPath).To(Equal(cfgFile.Name()))
 
@@ -164,6 +240,10 @@ var _ = Describe("Config Operations", func() {
 			Expect(Kubernetes.CACert).To(Equal(""))
 			Expect(Kubernetes.Token).To(Equal(""))
 			Expect(Kubernetes.APIServer).To(Equal("http://localhost:8080"))
+			Expect(Kubernetes.ServiceCIDR).To(Equal("172.16.1.0/24"))
+			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{mustParseCIDR("10.128.0.0/14"), 23},
+			}))
 
 			for _, a := range []OvnAuthConfig{OvnNorth, OvnSouth} {
 				Expect(a.Scheme).To(Equal(OvnDBSchemeUnix))
@@ -180,38 +260,33 @@ var _ = Describe("Config Operations", func() {
 
 	It("reads defaults from ovs-vsctl external IDs", func() {
 		app.Action = func(ctx *cli.Context) error {
+			fexec := ovntest.NewFakeExec()
+
 			// k8s-api-server
-			fakeCmds := ovntest.AddFakeCmd(nil, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-api-server",
 				Output: "https://somewhere.com:8081",
 			})
 
 			// k8s-api-token
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-api-token",
 				Output: "asadfasdfasrw3atr3r3rf33fasdaa3233",
 			})
 			// k8s-ca-certificate
-			fname, err := createTempFile("kube-cacert.pem")
+			fname, err := createTempFile("ca.crt")
 			Expect(err).NotTo(HaveOccurred())
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-ca-certificate",
 				Output: fname,
 			})
 			// ovn-nb address
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:ovn-nb",
 				Output: "tcp:1.1.1.1:6441",
 			})
 
-			fexec := &fakeexec.FakeExec{
-				CommandScript: fakeCmds,
-				LookPathFunc: func(file string) (string, error) {
-					return fmt.Sprintf("/fake-bin/%s", file), nil
-				},
-			}
-
-			cfgPath, err := InitConfig(ctx, fexec, &Defaults{
+			cfgPath, err := InitConfigSa(ctx, fexec, tmpDir, &Defaults{
 				OvnNorthAddress: true,
 				K8sAPIServer:    true,
 				K8sToken:        true,
@@ -219,7 +294,7 @@ var _ = Describe("Config Operations", func() {
 			})
 			Expect(err).NotTo(HaveOccurred())
 			Expect(cfgPath).To(Equal(cfgFile.Name()))
-			Expect(fexec.CommandCalls).To(Equal(len(fakeCmds)))
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue())
 
 			Expect(Kubernetes.APIServer).To(Equal("https://somewhere.com:8081"))
 			Expect(Kubernetes.CACert).To(Equal(fname))
@@ -245,38 +320,37 @@ var _ = Describe("Config Operations", func() {
 
 	It("reads defaults (multiple master) from ovs-vsctl external IDs", func() {
 		app.Action = func(ctx *cli.Context) error {
+			fexec := ovntest.NewFakeExec()
+
 			// k8s-api-server
-			fakeCmds := ovntest.AddFakeCmd(nil, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-api-server",
 				Output: "https://somewhere.com:8081",
 			})
 
 			// k8s-api-token
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-api-token",
 				Output: "asadfasdfasrw3atr3r3rf33fasdaa3233",
 			})
 			// k8s-ca-certificate
 			fname, err := createTempFile("kube-cacert.pem")
 			Expect(err).NotTo(HaveOccurred())
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:k8s-ca-certificate",
 				Output: fname,
 			})
 			// ovn-nb address
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd:    "ovs-vsctl --timeout=15 --if-exists get Open_vSwitch . external_ids:ovn-nb",
 				Output: "tcp:1.1.1.1:6441,tcp:1.1.1.2:6641,tcp:1.1.1.3:6641",
 			})
 
-			fexec := &fakeexec.FakeExec{
-				CommandScript: fakeCmds,
-				LookPathFunc: func(file string) (string, error) {
-					return fmt.Sprintf("/fake-bin/%s", file), nil
-				},
-			}
+			tokenFile, err1 := createTempFileContent("token", "TG9yZW0gaXBzdW0gZ")
+			Expect(err1).NotTo(HaveOccurred())
+			defer os.Remove(tokenFile)
 
-			cfgPath, err := InitConfig(ctx, fexec, &Defaults{
+			cfgPath, err := InitConfigSa(ctx, fexec, tmpDir, &Defaults{
 				OvnNorthAddress: true,
 				K8sAPIServer:    true,
 				K8sToken:        true,
@@ -284,7 +358,7 @@ var _ = Describe("Config Operations", func() {
 			})
 			Expect(err).NotTo(HaveOccurred())
 			Expect(cfgPath).To(Equal(cfgFile.Name()))
-			Expect(fexec.CommandCalls).To(Equal(len(fakeCmds)))
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue())
 
 			Expect(Kubernetes.APIServer).To(Equal("https://somewhere.com:8081"))
 			Expect(Kubernetes.CACert).To(Equal(fname))
@@ -309,6 +383,66 @@ var _ = Describe("Config Operations", func() {
 		Expect(err).NotTo(HaveOccurred())
 	})
 
+	It("uses serviceaccount files", func() {
+		kubeCAcertFile, err := createTempFile("ca.crt")
+		Expect(err).NotTo(HaveOccurred())
+		defer os.Remove(kubeCAcertFile)
+
+		tokenFile, err1 := createTempFileContent("token", "TG9yZW0gaXBzdW0gZ")
+		Expect(err1).NotTo(HaveOccurred())
+		defer os.Remove(tokenFile)
+
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfigSa(ctx, kexec.New(), tmpDir, nil)
+			Expect(err).NotTo(HaveOccurred())
+
+			Expect(Kubernetes.CACert).To(Equal(kubeCAcertFile))
+			Expect(Kubernetes.Token).To(Equal("TG9yZW0gaXBzdW0gZ"))
+
+			return nil
+		}
+		err2 := app.Run([]string{app.Name})
+		Expect(err2).NotTo(HaveOccurred())
+
+	})
+
+	It("uses environment variables", func() {
+		kubeconfigEnvFile, err := createTempFile("kubeconfig.env")
+		Expect(err).NotTo(HaveOccurred())
+		defer os.Remove(kubeconfigEnvFile)
+		os.Setenv("KUBECONFIG", kubeconfigEnvFile)
+		defer os.Setenv("KUBECONFIG", "")
+
+		os.Setenv("K8S_TOKEN", "this is the  token test")
+		defer os.Setenv("K8S_TOKEN", "")
+
+		os.Setenv("K8S_APISERVER", "https://9.2.3.4:6443")
+		defer os.Setenv("K8S_APISERVER", "")
+
+		kubeCAFile, err1 := createTempFile("kube-ca.crt")
+		Expect(err1).NotTo(HaveOccurred())
+		defer os.Remove(kubeCAFile)
+		os.Setenv("K8S_CACERT", kubeCAFile)
+		defer os.Setenv("K8S_CACERT", "")
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfigSa(ctx, kexec.New(), tmpDir, nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+
+			Expect(Kubernetes.Kubeconfig).To(Equal(kubeconfigEnvFile))
+			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
+			Expect(Kubernetes.Token).To(Equal("this is the  token test"))
+			Expect(Kubernetes.APIServer).To(Equal("https://9.2.3.4:6443"))
+
+			return nil
+		}
+		err = app.Run([]string{app.Name, "-config-file=" + cfgFile.Name()})
+		Expect(err).NotTo(HaveOccurred())
+
+	})
+
 	It("overrides defaults with config file options", func() {
 		kubeconfigFile, err := createTempFile("kubeconfig")
 		Expect(err).NotTo(HaveOccurred())
@@ -318,37 +452,7 @@ var _ = Describe("Config Operations", func() {
 		Expect(err).NotTo(HaveOccurred())
 		defer os.Remove(kubeCAFile)
 
-		cfgData := fmt.Sprintf(`[default]
-mtu=1500
-conntrack-zone=64321
-
-[kubernetes]
-kubeconfig=%s
-apiserver=https://1.2.3.4:6443
-token=TG9yZW0gaXBzdW0gZ
-cacert=%s
-
-[logging]
-loglevel=5
-logfile=/var/log/ovnkube.log
-
-[cni]
-conf-dir=/etc/cni/net.d22
-plugin=ovn-k8s-cni-overlay22
-
-[ovnnorth]
-address=ssl://1.2.3.4:6641
-client-privkey=/path/to/nb-client-private.key
-client-cert=/path/to/nb-client.crt
-client-cacert=/path/to/nb-client-ca.crt
-
-[ovnsouth]
-address=ssl://1.2.3.4:6642
-client-privkey=/path/to/sb-client-private.key
-client-cert=/path/to/sb-client.crt
-client-cacert=/path/to/sb-client-ca.crt
-`, kubeconfigFile, kubeCAFile)
-		err = ioutil.WriteFile(cfgFile.Name(), []byte(cfgData), 0644)
+		err = writeTestConfigFile(cfgFile.Name(), "kubeconfig="+kubeconfigFile, "cacert="+kubeCAFile)
 		Expect(err).NotTo(HaveOccurred())
 
 		app.Action = func(ctx *cli.Context) error {
@@ -367,6 +471,10 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("TG9yZW0gaXBzdW0gZ"))
 			Expect(Kubernetes.APIServer).To(Equal("https://1.2.3.4:6443"))
+			Expect(Kubernetes.ServiceCIDR).To(Equal("172.18.0.0/24"))
+			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{mustParseCIDR("10.129.0.0/14"), 23},
+			}))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
 			Expect(OvnNorth.PrivKey).To(Equal("/path/to/nb-client-private.key"))
@@ -380,6 +488,12 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(OvnSouth.CACert).To(Equal("/path/to/sb-client-ca.crt"))
 			Expect(OvnSouth.Address).To(Equal("ssl:1.2.3.4:6642"))
 
+			Expect(Gateway.Mode).To(Equal(GatewayModeShared))
+			Expect(Gateway.Interface).To(Equal("eth1"))
+			Expect(Gateway.NextHop).To(Equal("1.3.4.5"))
+			Expect(Gateway.VLANID).To(Equal(uint(10)))
+			Expect(Gateway.NodeportEnable).To(BeFalse())
+
 			return nil
 		}
 		err = app.Run([]string{app.Name, "-config-file=" + cfgFile.Name()})
@@ -395,36 +509,7 @@ client-cacert=/path/to/sb-client-ca.crt
 		Expect(err).NotTo(HaveOccurred())
 		defer os.Remove(kubeCAFile)
 
-		err = ioutil.WriteFile(cfgFile.Name(), []byte(`[default]
-mtu=1500
-conntrack-zone=64321
-
-[kubernetes]
-kubeconfig=/path/to/kubeconfig
-apiserver=https://1.2.3.4:6443
-token=TG9yZW0gaXBzdW0gZ
-cacert=/path/to/kubeca.crt
-
-[logging]
-loglevel=5
-logfile=/var/log/ovnkube.log
-
-[cni]
-conf-dir=/etc/cni/net.d22
-plugin=ovn-k8s-cni-overlay22
-
-[ovnnorth]
-address=ssl://1.2.3.4:6641
-client-privkey=/path/to/nb-client-private.key
-client-cert=/path/to/nb-client.crt
-client-cacert=/path/to/nb-client-ca.crt
-
-[ovnsouth]
-address=ssl://1.2.3.4:6642
-client-privkey=/path/to/sb-client-private.key
-client-cert=/path/to/sb-client.crt
-client-cacert=/path/to/sb-client-ca.crt
-`), 0644)
+		err = writeTestConfigFile(cfgFile.Name())
 		Expect(err).NotTo(HaveOccurred())
 
 		app.Action = func(ctx *cli.Context) error {
@@ -443,6 +528,10 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("asdfasdfasdfasfd"))
 			Expect(Kubernetes.APIServer).To(Equal("https://4.4.3.2:8080"))
+			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
+			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{mustParseCIDR("10.130.0.0/15"), 24},
+			}))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
 			Expect(OvnNorth.PrivKey).To(Equal("/client/privkey"))
@@ -456,6 +545,11 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(OvnSouth.CACert).To(Equal("/client/cacert2"))
 			Expect(OvnSouth.Address).To(Equal("ssl:6.5.4.1:6652"))
 
+			Expect(Gateway.Mode).To(Equal(GatewayModeSpare))
+			Expect(Gateway.Interface).To(Equal("eth5"))
+			Expect(Gateway.NextHop).To(Equal("1.3.5.6"))
+			Expect(Gateway.VLANID).To(Equal(uint(100)))
+			Expect(Gateway.NodeportEnable).To(BeTrue())
 			return nil
 		}
 		cliArgs := []string{
@@ -467,10 +561,12 @@ client-cacert=/path/to/sb-client-ca.crt
 			"-logfile=/some/logfile",
 			"-cni-conf-dir=/some/cni/dir",
 			"-cni-plugin=a-plugin",
+			"-cluster-subnets=10.130.0.0/15/24",
 			"-k8s-kubeconfig=" + kubeconfigFile,
 			"-k8s-apiserver=https://4.4.3.2:8080",
 			"-k8s-cacert=" + kubeCAFile,
 			"-k8s-token=asdfasdfasdfasfd",
+			"-k8s-service-cidr=172.15.0.0/24",
 			"-nb-address=ssl://6.5.4.3:6651",
 			"-nb-client-privkey=/client/privkey",
 			"-nb-client-cert=/client/cert",
@@ -479,11 +575,177 @@ client-cacert=/path/to/sb-client-ca.crt
 			"-sb-client-privkey=/client/privkey2",
 			"-sb-client-cert=/client/cert2",
 			"-sb-client-cacert=/client/cacert2",
+			"-gateway-mode=spare",
+			"-gateway-interface=eth5",
+			"-gateway-nexthop=1.3.5.6",
+			"-gateway-vlanid=100",
+			"-nodeport",
 		}
 		err = app.Run(cliArgs)
 		Expect(err).NotTo(HaveOccurred())
 	})
 
+	It("overrides config file and defaults with CLI legacy service-cluster-ip-range option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[kubernetes]
+service-cidr=172.18.0.0/24
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+			"-service-cluster-ip-range=172.15.0.0/24",
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("returns an error when the k8s-service-cidr is invalid", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("kubernetes service network CIDR \"adsfasdfaf\" invalid: invalid CIDR address: adsfasdfaf"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-k8s-service-cidr=adsfasdfaf",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("overrides config file and defaults with CLI legacy cluster-subnet option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[default]
+cluster-subnets=172.18.0.0/24
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{mustParseCIDR("172.15.0.0/24"), 24},
+			}))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+			"-cluster-subnet=172.15.0.0/24",
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("returns an error when the cluster-subnets is invalid", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("cluster subnet invalid: CIDR \"adsfasdfaf\" not properly formatted"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=adsfasdfaf",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("overrides config file and defaults with CLI legacy --init-gateways option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[gateway]
+mode=spare
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Gateway.Mode).To(Equal(GatewayModeShared))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+			"-init-gateways",
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("overrides config file and defaults with CLI legacy --gateway-spare-interface option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[gateway]
+mode=shared
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Gateway.Mode).To(Equal(GatewayModeSpare))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+			"-init-gateways",
+			"-gateway-spare-interface",
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("overrides config file and defaults with CLI legacy --gateway-local option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[gateway]
+mode=shared
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Gateway.Mode).To(Equal(GatewayModeLocal))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+			"-init-gateways",
+			"-gateway-local",
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("returns an error when the gateway mode is invalid", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("invalid gateway mode \"adsfasdfaf\": expect one of shared,spare,local"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-gateway-mode=adsfasdfaf",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
 	It("overrides config file and defaults with CLI options (multi-master)", func() {
 		kubeconfigFile, err := createTempFile("kubeconfig")
 		Expect(err).NotTo(HaveOccurred())
@@ -493,36 +755,7 @@ client-cacert=/path/to/sb-client-ca.crt
 		Expect(err).NotTo(HaveOccurred())
 		defer os.Remove(kubeCAFile)
 
-		err = ioutil.WriteFile(cfgFile.Name(), []byte(`[default]
-mtu=1500
-conntrack-zone=64321
-
-[kubernetes]
-kubeconfig=/path/to/kubeconfig
-apiserver=https://1.2.3.4:6443
-token=TG9yZW0gaXBzdW0gZ
-cacert=/path/to/kubeca.crt
-
-[logging]
-loglevel=5
-logfile=/var/log/ovnkube.log
-
-[cni]
-conf-dir=/etc/cni/net.d22
-plugin=ovn-k8s-cni-overlay22
-
-[ovnnorth]
-address=ssl://1.2.3.4:6641
-client-privkey=/path/to/nb-client-private.key
-client-cert=/path/to/nb-client.crt
-client-cacert=/path/to/nb-client-ca.crt
-
-[ovnsouth]
-address=ssl://1.2.3.4:6642
-client-privkey=/path/to/sb-client-private.key
-client-cert=/path/to/sb-client.crt
-client-cacert=/path/to/sb-client-ca.crt
-`), 0644)
+		err = writeTestConfigFile(cfgFile.Name())
 		Expect(err).NotTo(HaveOccurred())
 
 		app.Action = func(ctx *cli.Context) error {
@@ -541,6 +774,7 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("asdfasdfasdfasfd"))
 			Expect(Kubernetes.APIServer).To(Equal("https://4.4.3.2:8080"))
+			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
 			Expect(OvnNorth.PrivKey).To(Equal("/client/privkey"))
@@ -571,6 +805,7 @@ client-cacert=/path/to/sb-client-ca.crt
 			"-k8s-apiserver=https://4.4.3.2:8080",
 			"-k8s-cacert=" + kubeCAFile,
 			"-k8s-token=asdfasdfasdfasfd",
+			"-k8s-service-cidr=172.15.0.0/24",
 			"-nb-address=ssl://6.5.4.3:6651,ssl://6.5.4.4:6651,ssl://6.5.4.5:6651",
 			"-nb-client-privkey=/client/privkey",
 			"-nb-client-cert=/client/cert",
@@ -612,19 +847,11 @@ client-cacert=/path/to/sb-client-ca.crt
 		It("configures client northbound SSL correctly", func() {
 			const nbURLOVN string = "ssl:1.2.3.4:6641"
 
-			fakeCmds := ovntest.AddFakeCmd(nil, &ovntest.ExpectedCmd{
-				Cmd: fmt.Sprintf("ovn-nbctl --db=%s --timeout=5 --private-key=%s --certificate=%s --bootstrap-ca-cert=%s list nb_global", nbURLOVN, keyFile, certFile, caFile),
+			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmdsNoOutputNoError([]string{
+				"ovn-nbctl --db=" + nbURLOVN + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"" + nbURLOVN + "\"",
 			})
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
-				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"%s\"", nbURLOVN),
-			})
-
-			fexec := &fakeexec.FakeExec{
-				CommandScript: fakeCmds,
-				LookPathFunc: func(file string) (string, error) {
-					return fmt.Sprintf("/fake-bin/%s", file), nil
-				},
-			}
 
 			cliConfig := &OvnAuthConfig{
 				Address: nbURL,
@@ -645,31 +872,19 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(a.GetURL()).To(Equal(nbURLOVN))
 			err = a.SetDBAuth()
 			Expect(err).NotTo(HaveOccurred())
-			Expect(fexec.CommandCalls).To(Equal(len(fakeCmds)))
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue())
 		})
 
 		It("configures client southbound SSL correctly", func() {
 			const sbURLOVN string = "ssl:1.2.3.4:6642"
 
-			fakeCmds := ovntest.AddFakeCmd(nil, &ovntest.ExpectedCmd{
-				Cmd: fmt.Sprintf("ovn-nbctl --db=%s --timeout=5 --private-key=%s --certificate=%s --bootstrap-ca-cert=%s list nb_global", sbURLOVN, keyFile, certFile, caFile),
+			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmdsNoOutputNoError([]string{
+				"ovn-nbctl --db=" + sbURLOVN + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
+				"ovs-vsctl --timeout=15 del-ssl",
+				"ovs-vsctl --timeout=15 set-ssl " + keyFile + " " + certFile + " " + caFile,
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-remote=\"" + sbURLOVN + "\"",
 			})
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
-				Cmd: "ovs-vsctl --timeout=15 del-ssl",
-			})
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
-				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set-ssl %s %s %s", keyFile, certFile, caFile),
-			})
-			fakeCmds = ovntest.AddFakeCmd(fakeCmds, &ovntest.ExpectedCmd{
-				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-remote=\"%s\"", sbURLOVN),
-			})
-
-			fexec := &fakeexec.FakeExec{
-				CommandScript: fakeCmds,
-				LookPathFunc: func(file string) (string, error) {
-					return fmt.Sprintf("/fake-bin/%s", file), nil
-				},
-			}
 
 			cliConfig := &OvnAuthConfig{
 				Address: sbURL,
@@ -690,7 +905,7 @@ client-cacert=/path/to/sb-client-ca.crt
 			Expect(a.GetURL()).To(Equal(sbURLOVN))
 			err = a.SetDBAuth()
 			Expect(err).NotTo(HaveOccurred())
-			Expect(fexec.CommandCalls).To(Equal(len(fakeCmds)))
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue())
 		})
 	})
 
